#include <stdio.h>
#include <time.h>
#include <curses.h>
#include <ncurses.h>
#include <string.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <sys/types.h>
#include "defs.h"

void check_watches(int min);
char *parse_extalarm(const char *exta, int i, int j, char *output);
void do_alarms(int alarmnum, int i, int j);
void edit_tvconf(void);

int main(void) { 

    time_t t = time(NULL);  
    struct tm *tid;
    char c;
    int    wwopen=0;
    WINDOW *win_reload;
    WINDOW *win_watch;

    printf("tvprog %s\n",VERSION); 
    printf("Loading lists...\n");
    fflush(stdout);

    reload(1); //bootup
    init_curses();
    init_colors();
    print_border();
    halfdelay(10);
    print_all(wwopen, win_watch);

    while(1) {
        check_resize(wwopen, win_watch);

        if(t+60 < time(NULL)) {
            print_all(wwopen, win_watch);
            t = time(NULL);
            tid = localtime(&t);


            if (((tid->tm_hour)%24) == 6 && tid->tm_min == 0) { // 06:00 AM, update!
                reload(0); //not bootup
                init_colors();
                print_border();
                print_all(wwopen, win_watch);
            }

            check_watches(((tid->tm_hour)%24)*60+tid->tm_min);
        }
        if ((c = getch()) != ERR) {
            //mvprintw(20,10,"Got key %d   ",c);           
            switch (c) {
                case 114:
                case 82:
                    win_reload = print_reload_win();  
                    reload(1);
                    kill_win(win_reload);
                    init_colors();
                    print_border();
                    print_all(wwopen, win_watch);
                    break;
                case 113:
                case 81:
                    kill_curses();
                    exit(0);
                    break;
                case 119:
                case 87:
                    if (wwopen == 0 ) {
                        wwopen=1; 
                        if ((win_watch = print_watch_win()) == NULL) wwopen=0;  // if the window fails to open
                    }
                    else if (wwopen == 1 ) {
                        kill_win(win_watch);
                        wwopen=0;
                        print_border();
                        print_all(wwopen, win_watch);
                    }
                    break;
                case 101:
                case 69:
                    edit_tvconf();
                    win_reload = print_reload_win();  
                    reload(1);
                    kill_win(win_reload);
                    init_colors();
                    print_border();
                    print_all(wwopen, win_watch);
                    break;
            
            }
        }  
    }  
}

void check_watches(int min) {
    
    int i,j;

    for (i = 0; i < 16 && kanaler[i].used == 1; i++) {
        for (j = 0; j < MAXCHAN && kanaler[i].program[j][0] != ' '; j++) { // empty 
            if (kanaler[i].watch[j] == 1 && alarm1 != 0 && (kanaler[i].starttid[j] - min) > 0) { 
                if (kanaler[i].watch[j] == 1 && alarm1 != 0 && kanaler[i].starttid[j] - min == alarm1) {
                    do_alarms(1,i,j); // 1 == alarm1, i == channel, j == program 
                }
            }
            else if (kanaler[i].watch[j] == 1 && alarm1 != 0 && kanaler[i].starttid[j] + 1440 - min  == alarm1) {
                do_alarms(1,i,j);
            }
            if (kanaler[i].watch[j] == 1 && alarm2 != 0 && (kanaler[i].starttid[j] - min) > 0) { 
                if (kanaler[i].watch[j] == 1 && alarm2 != 0 && kanaler[i].starttid[j] - min == alarm2) {
                    do_alarms(2,i,j); // 1 == alarm1, i == channel, j == program 
                }
            }
            else if (kanaler[i].watch[j] == 1 && alarm2 != 0 && kanaler[i].starttid[j] + 1440 - min  == alarm2) {
                do_alarms(2,i,j);
            }
        }
    }
}

void do_alarms(int alarmnum, int i, int j) {

    char ex_alarm1[256];
    char ex_alarm2[256];

    if (alarmnum == 1) {
        if (extalarm1[0] == '\0') {
            beep();
        }
        else {
            system(parse_extalarm(extalarm1,i,j,ex_alarm1)); 
        }
    }
    else if (alarmnum == 2) {
        if (extalarm2[0] == '\0') {
            beep();
        }
        else {
            system(parse_extalarm(extalarm2,i,j,ex_alarm2));
        }
    }
}

char *parse_extalarm(const char *input, int i, int j, char *output) {   // fulhack?? no, no, no!

    char starttime[6], stoptime[6];
    char tmpstr[256];
    char *p;
    char *q;


    memcpy(output,input,256);
    memcpy(tmpstr,input,256);
    if ((p = strstr(output,"%c")) != NULL) {                            // channel
        memcpy(p,kanaler[i].name,strlen(kanaler[i].name));
        p +=strlen(kanaler[i].name);
        q = strstr(tmpstr,"%c");
        q += 2;
        memcpy(p,q,strlen(q));
    }
    memcpy(tmpstr,output,256);
    if ((p = strstr(output,"%t")) != NULL) {                           // start time
        sprintf(starttime,"%.2d:%.2d",(kanaler[i].starttid[j] - kanaler[i].starttid[j]%60)/60, kanaler[i].starttid[j]%60);
        memcpy(p,starttime,5);
        p +=5;
        q = strstr(tmpstr,"%t");
        q += 2;
        memcpy(p,q,strlen(q));
    }
    memcpy(tmpstr,output,256);
    if ((p = strstr(output,"%s")) != NULL) {                           // stop time
        sprintf(stoptime,"%.2d:%.2d",(kanaler[i].starttid[j+1] - kanaler[i].starttid[j+1]%60)/60, kanaler[i].starttid[j+1]%60);
        memcpy(p,stoptime,5);
        p +=5;
        q = strstr(tmpstr,"%s");
        q += 2;
        memcpy(p,q,strlen(q));
    }
    memcpy(tmpstr,output,256);
    if ((p = strstr(output,"%p")) != NULL) {                          // program
        memcpy(p,kanaler[i].program[j],strlen(kanaler[i].program[j]));
        p +=strlen(kanaler[i].program[j]);
        q = strstr(tmpstr,"%p");
        q += 2;
        memcpy(p,q,strlen(q));
    }
    return output;       
}

void edit_tvconf(void) {

    char   conf[512];
    
    kill_curses();
    memset(&conf, 0, sizeof(conf));
    snprintf(conf, sizeof(conf)-1, "%s %s/.tvconfig", getenv("EDITOR"), getenv("HOME"));
    system(conf); 
    init_curses(); 

}

    
